
#include <string>
#include <boost/multi_index_container.hpp>
#include <boost/multi_index/hashed_index.hpp>
#include <boost/multi_index/ordered_index.hpp>
#include <boost/multi_index/identity.hpp>
#include <boost/multi_index/member.hpp>

using boost::multi_index_container;
using namespace boost::multi_index;

struct employee
{
  int         id;
  std::string  name;
  int         ssnumber;

    employee(int id,const std::string& name,int ssnumber):
    id(id),name(name),ssnumber(ssnumber){}

  bool operator<(const employee& e)const{return id<e.id;}
};

struct ustring_hasher {

    std::size_t operator ()(std::string const& wstr) const {
        boost::hash<std::string> hasher;
        return hasher(wstr);
    }
};

struct name{} ; // just for boost tags

typedef multi_index_container<
  employee,
  indexed_by<
    // hashed on name
      hashed_unique<tag<name>, member<employee,std::string,&employee::name>, ustring_hasher >
   >
    > employee_set;

typedef employee_set::index<name>::type employee_set_by_name;


#define StrConst1 "Foobar!@##$%%^&Yavapai@QuuxQUUX@@27"
#define StrConst2 "CabbagesAndTurnips$haveDrivenMeAway"

void testBasic() {
        employee_set testSet;
        std::string s(StrConst1);
        employee e1(1, s, 100);
        testSet.insert(e1);

        s = std::string(StrConst2);
        employee e2(2, s, 101);
        testSet.insert(e2);

        employee_set_by_name const& name_index = testSet.get<name>();

        s = std::string(StrConst1);

        employee_set::index_iterator<name>::type it = name_index.find(s);

        s = std::string("Hello There");
        it = name_index.find(s);

}

